@@ -210,8 +210,8 @@ public class GitResourceVariantComparatorTest extends GitTestCase {
 	public void shouldReturnFalseWhenContentLengthIsDifferent()
 			throws Exception {
 		// when
-		byte[] shortContent = "short content".getBytes();
-		byte[] longContent = "very long long content".getBytes();
+		byte[] shortContent = "short content".getBytes("UTF-8");
+		byte[] longContent = "very long long content".getBytes("UTF-8");
 		GitSynchronizeData data = new GitSynchronizeData(repo, HEAD, HEAD, true);
 		GitSynchronizeDataSet dataSet = new GitSynchronizeDataSet(data);
 		GitResourceVariantComparator grvc = new GitResourceVariantComparator(
@@ -247,9 +247,9 @@ public class GitResourceVariantComparatorTest extends GitTestCase {
 	@SuppressWarnings("boxing")
 	public void shouldReturnFalseWhenShortContentIsDifferent() throws Exception {
 		// when
-		byte[] localContent = "very long long content".getBytes();
+		byte[] localContent = "very long long content".getBytes("UTF-8");
 		// this typo should be here
-		byte[] remoteContent = "very long lonk content".getBytes();
+		byte[] remoteContent = "very long lonk content".getBytes("UTF-8");
 		GitSynchronizeData data = new GitSynchronizeData(repo, HEAD, HEAD, true);
 		GitSynchronizeDataSet dataSet = new GitSynchronizeDataSet(data);
 		GitResourceVariantComparator grvc = new GitResourceVariantComparator(
@@ -369,8 +369,8 @@ public class GitResourceVariantComparatorTest extends GitTestCase {
 	@SuppressWarnings("boxing")
 	public void shouldReturnTrueWhenShortContentIsDifferent() throws Exception {
 		// when
-		byte[] localContent = "very long long content".getBytes();
-		byte[] remoteContent = "very long long content".getBytes();
+		byte[] localContent = "very long long content".getBytes("UTF-8");
+		byte[] remoteContent = "very long long content".getBytes("UTF-8");
 		GitSynchronizeData data = new GitSynchronizeData(repo, HEAD, HEAD, true);
 		GitSynchronizeDataSet dataSet = new GitSynchronizeDataSet(data);
 		GitResourceVariantComparator grvc = new GitResourceVariantComparator(
