@@ -182,8 +182,10 @@ public class CompareUtils {
 
 
 	/**
-	 * Creates a {@link ITypedElement} for the commit which is the common ancestor of
-	 * the provided commits.
+	 * Creates a {@link ITypedElement} for the commit which is the common
+	 * ancestor of the provided commits. Returns null if no such commit exists
+	 * or if {@code gitPath} is not contained in the common ancestor
+	 *
 	 * @param gitPath
 	 *            path within the ancestor commit's tree of the file.
 	 * @param commit1
@@ -204,9 +206,12 @@ public class CompareUtils {
 			Activator.logError(NLS.bind(UIText.CompareUtils_errorCommonAncestor,
 					commit1.getName(), commit2.getName()), e);
 		}
-		if (commonAncestor != null)
-			ancestor = CompareUtils
-				.getFileRevisionTypedElement(gitPath, commonAncestor, db);
+		if (commonAncestor != null) {
+			ITypedElement ancestorCandidate = CompareUtils
+					.getFileRevisionTypedElement(gitPath, commonAncestor, db);
+			if (!(ancestorCandidate instanceof EmptyTypedElement))
+				ancestor = ancestorCandidate;
+		}
 		return ancestor;
 	}
 /**
@@ -635,21 +640,17 @@ public class CompareUtils {
 			Repository repository, final String gitPath, String srcRev,
 			String dstRev) {
 		ITypedElement ancestor = null;
-		RevCommit commonAncestor = null;
 		try {
 			final ObjectId srcID = repository.resolve(srcRev);
 			final ObjectId dstID = repository.resolve(dstRev);
 			if (srcID != null && dstID != null)
-				commonAncestor = RevUtils.getCommonAncestor(repository, srcID,
-						dstID);
+				ancestor = getFileRevisionTypedElementForCommonAncestor(
+						gitPath, srcID, dstID, repository);
 		} catch (IOException e) {
 			Activator
 					.logError(NLS.bind(UIText.CompareUtils_errorCommonAncestor,
 							srcRev, dstRev), e);
 		}
-		if (commonAncestor != null)
-			ancestor = CompareUtils.getFileRevisionTypedElement(gitPath,
-					commonAncestor, repository);
 		return ancestor;
 	}
 
