@@ -10,8 +10,11 @@
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.commands.shared;
 
+import org.eclipse.core.commands.ExecutionException;
+import org.eclipse.egit.core.op.RebaseOperation;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.jgit.api.RebaseCommand.Operation;
+import org.eclipse.jgit.lib.Repository;
 
 /**
  * Implements "Abort Rebase"
@@ -21,7 +24,13 @@ public class AbortRebaseCommand extends AbstractRebaseCommandHandler {
 	 * Default constructor
 	 */
 	public AbortRebaseCommand() {
-		super(Operation.ABORT, UIText.AbortRebaseCommand_JobName,
+		super(UIText.AbortRebaseCommand_JobName,
 				UIText.AbortRebaseCommand_CancelDialogMessage);
 	}
+
+	@Override
+	protected RebaseOperation createRebaseOperation(Repository repository)
+			throws ExecutionException {
+		return new RebaseOperation(repository, Operation.ABORT);
+	}
 }
