@@ -20,14 +20,18 @@ import java.util.List;
 import java.util.TreeMap;
 import java.util.regex.Pattern;
 
+import org.eclipse.egit.core.securestorage.EGitSecureStore;
 import org.eclipse.egit.core.securestorage.UserPasswordCredentials;
 import org.eclipse.egit.ui.Activator;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.UIUtils.IPreviousValueProposalHandler;
 import org.eclipse.egit.ui.internal.UIText;
+import org.eclipse.egit.ui.internal.components.RemoteSelectionCombo.IRemoteSelectionListener;
+import org.eclipse.egit.ui.internal.components.RemoteSelectionCombo.SelectionType;
 import org.eclipse.egit.ui.internal.provisional.wizards.GitRepositoryInfo;
 import org.eclipse.egit.ui.internal.provisional.wizards.IRepositorySearchResult;
+import org.eclipse.equinox.security.storage.StorageException;
 import org.eclipse.jface.dialogs.Dialog;
 import org.eclipse.jface.layout.GridDataFactory;
 import org.eclipse.jface.preference.IPreferenceStore;
@@ -72,8 +76,6 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 
 	private final static String USED_URIS_PREF = "RepositorySelectionPage.UsedUris"; //$NON-NLS-1$
 
-	private static final int REMOTE_CONFIG_TEXT_MAX_LENGTH = 80;
-
 	private final List<RemoteConfig> configuredRemotes;
 
 	private final boolean sourceSelection;
@@ -110,7 +112,7 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 
 	private Button remoteButton;
 
-	private Combo remoteCombo;
+	private RemoteSelectionCombo remoteCombo;
 
 	private Composite uriPanel;
 
@@ -344,7 +346,6 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 		this.presetUri = preset;
 
 		this.configuredRemotes = getUsableConfigs(configuredRemotes);
-		this.remoteConfig = selectDefaultRemoteConfig();
 
 		selection = RepositorySelection.INVALID_SELECTION;
 
@@ -409,6 +410,8 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 
 		if (configuredRemotes != null)
 			createRemotePanel(panel);
+		else
+			createRemoteNamePanel(panel);
 
 		createUriPanel(panel);
 
@@ -444,24 +447,25 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 		gd.horizontalAlignment = SWT.FILL;
 		remotePanel.setLayoutData(gd);
 
-		remoteCombo = new Combo(remotePanel, SWT.READ_ONLY | SWT.DROP_DOWN);
-		final String items[] = new String[configuredRemotes.size()];
-		int i = 0;
-		for (final RemoteConfig rc : configuredRemotes)
-			items[i++] = getTextForRemoteConfig(rc);
-		final int defaultIndex = configuredRemotes.indexOf(remoteConfig);
-		remoteCombo.setItems(items);
-		remoteCombo.select(defaultIndex);
-		remoteCombo.addSelectionListener(new SelectionAdapter() {
-			@Override
-			public void widgetSelected(SelectionEvent e) {
-				final int idx = remoteCombo.getSelectionIndex();
-				remoteConfig = configuredRemotes.get(idx);
+		SelectionType selectionType = sourceSelection ? SelectionType.FETCH : SelectionType.PUSH;
+		remoteCombo = new RemoteSelectionCombo(remotePanel, SWT.NULL, selectionType);
+		remoteConfig = remoteCombo.setItems(configuredRemotes);
+		remoteCombo.addRemoteSelectionListener(new IRemoteSelectionListener() {
+			public void remoteSelected(RemoteConfig rc) {
+				remoteConfig = rc;
 				checkPage();
 			}
 		});
 	}
 
+	/**
+	 *
+	 * @param panel
+	 */
+	protected void createRemoteNamePanel(Composite panel) {
+		// Only used by subclass
+	}
+
 	private void createUriPanel(final Composite parent) {
 		if (configuredRemotes != null) {
 			uriButton = new Button(parent, SWT.RADIO);
@@ -732,46 +736,10 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 		return null;
 	}
 
-	private RemoteConfig selectDefaultRemoteConfig() {
-		if (configuredRemotes == null)
-			return null;
-		for (final RemoteConfig rc : configuredRemotes)
-			if (Constants.DEFAULT_REMOTE_NAME.equals(rc.getName()))
-				return rc;
-		return configuredRemotes.get(0);
-	}
-
-	private String getTextForRemoteConfig(final RemoteConfig rc) {
-		final StringBuilder sb = new StringBuilder(rc.getName());
-		sb.append(": "); //$NON-NLS-1$
-		boolean first = true;
-		List<URIish> uris;
-		if (sourceSelection)
-			uris = rc.getURIs();
-		else {
-			uris = rc.getPushURIs();
-			// if no push URIs are defined, use fetch URIs instead
-			if (uris.isEmpty())
-				uris = rc.getURIs();
-		}
-
-		for (final URIish u : uris) {
-			final String uString = u.toString();
-			if (first)
-				first = false;
-			else {
-				sb.append(", "); //$NON-NLS-1$
-				if (sb.length() + uString.length() > REMOTE_CONFIG_TEXT_MAX_LENGTH) {
-					sb.append("..."); //$NON-NLS-1$
-					break;
-				}
-			}
-			sb.append(uString);
-		}
-		return sb.toString();
-	}
-
-	private void checkPage() {
+	/**
+	 * Check the user input and set messages in case of invalid input.
+	 */
+	protected void checkPage() {
 		if (isURISelected()) {
 			assert uri != null;
 			if (uriText.getText().length() == 0) {
@@ -848,6 +816,23 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 					}
 				}
 
+				if (Protocol.HTTP.handles(finalURI)
+						|| Protocol.HTTPS.handles(finalURI)) {
+					UserPasswordCredentials credentials = getSecureStoreCredentials(finalURI);
+					if (credentials != null) {
+						String u = credentials.getUser();
+						String p = credentials.getPassword();
+						String uriUser = finalURI.getUser();
+						if (uriUser == null) {
+							if (setSafeUser(u) || setSafePassword(p))
+								setStoreInSecureStore(true);
+						} else if (uriUser.length() != 0 && uriUser.equals(u)) {
+							if (setSafePassword(p))
+								setStoreInSecureStore(true);
+						}
+					}
+				}
+
 				selectionComplete(finalURI, null);
 				return;
 			} catch (URISyntaxException e) {
@@ -867,6 +852,40 @@ public class RepositorySelectionPage extends WizardPage implements IRepositorySe
 		}
 	}
 
+	private boolean setSafePassword(String p) {
+		if ((password == null || password.length() == 0) && p != null
+				&& p.length() != 0) {
+			password = p;
+			passText.setText(p);
+			return true;
+		}
+		return false;
+	}
+
+	private boolean setSafeUser(String u) {
+		if ((user == null || user.length() == 0) && u != null
+				&& u.length() != 0) {
+			user = u;
+			userText.setText(u);
+			return true;
+		}
+		return false;
+	}
+
+	private void setStoreInSecureStore(boolean store) {
+		storeInSecureStore = store;
+		storeCheckbox.setSelection(store);
+	}
+
+	private UserPasswordCredentials getSecureStoreCredentials(
+			final URIish finalURI) throws StorageException {
+		EGitSecureStore secureStore = org.eclipse.egit.core.Activator
+				.getDefault().getSecureStore();
+		UserPasswordCredentials credentials = secureStore
+				.getCredentials(finalURI);
+		return credentials;
+	}
+
 	private String unamp(String s) {
 		return s.replace("&", EMPTY_STRING); //$NON-NLS-1$
 	}
