@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2011, 2013 SAP AG and others.
+ * Copyright (c) 2011, 2014 SAP AG and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -9,6 +9,7 @@
  * Contributors:
  *    Mathias Kinzler (SAP AG) - initial implementation
  *    Robin Stocker <robin@nibor.org> - ignore linked resources
+ *    Robin Stocker <robin@nibor.org> - Unify workbench and PathNode tree code
  *******************************************************************************/
 package org.eclipse.egit.ui.internal.dialogs;
 
@@ -17,27 +18,22 @@ import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
 import java.util.ArrayList;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
 
 import org.eclipse.compare.ITypedElement;
 import org.eclipse.core.resources.IContainer;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.ResourcesPlugin;
-import org.eclipse.core.runtime.Assert;
-import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.core.runtime.Platform;
-import org.eclipse.core.runtime.PlatformObject;
 import org.eclipse.egit.core.AdaptableFileTreeIterator;
 import org.eclipse.egit.core.internal.CompareCoreUtils;
 import org.eclipse.egit.core.internal.storage.GitFileRevision;
+import org.eclipse.egit.core.internal.storage.WorkingTreeFileRevision;
 import org.eclipse.egit.core.internal.util.ResourceUtil;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
@@ -45,10 +41,10 @@ import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIUtils;
 import org.eclipse.egit.ui.internal.CompareUtils;
 import org.eclipse.egit.ui.internal.EgitUiEditorUtils;
-import org.eclipse.egit.ui.internal.FileEditableRevision;
 import org.eclipse.egit.ui.internal.FileRevisionTypedElement;
 import org.eclipse.egit.ui.internal.GitCompareFileRevisionEditorInput;
 import org.eclipse.egit.ui.internal.LocalFileRevision;
+import org.eclipse.egit.ui.internal.ResourceEditableRevision;
 import org.eclipse.egit.ui.internal.UIIcons;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.actions.BooleanPrefAction;
@@ -64,8 +60,6 @@ import org.eclipse.jface.layout.GridDataFactory;
 import org.eclipse.jface.layout.GridLayoutFactory;
 import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.jface.preference.IPersistentPreferenceStore;
-import org.eclipse.jface.resource.ImageDescriptor;
-import org.eclipse.jface.viewers.ArrayContentProvider;
 import org.eclipse.jface.viewers.BaseLabelProvider;
 import org.eclipse.jface.viewers.ILabelProvider;
 import org.eclipse.jface.viewers.IOpenListener;
@@ -76,7 +70,9 @@ import org.eclipse.jface.viewers.ITreeSelection;
 import org.eclipse.jface.viewers.OpenEvent;
 import org.eclipse.jface.viewers.StructuredSelection;
 import org.eclipse.jface.viewers.TreeViewer;
+import org.eclipse.jface.viewers.Viewer;
 import org.eclipse.jface.viewers.ViewerComparator;
+import org.eclipse.jface.viewers.ViewerFilter;
 import org.eclipse.jgit.dircache.DirCacheIterator;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.Ref;
@@ -96,13 +92,12 @@ import org.eclipse.swt.SWT;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Menu;
+import org.eclipse.team.core.history.IFileRevision;
 import org.eclipse.ui.ISharedImages;
 import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchWindow;
 import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
-import org.eclipse.ui.model.IWorkbenchAdapter;
-import org.eclipse.ui.model.WorkbenchContentProvider;
 import org.eclipse.ui.model.WorkbenchLabelProvider;
 import org.eclipse.ui.navigator.ICommonMenuConstants;
 import org.eclipse.ui.part.IShowInSource;
@@ -113,10 +108,9 @@ import org.eclipse.ui.part.ViewPart;
  * Shows a tree when opening compare on an {@link IContainer}, or a
  * {@link Repository}
  * <p>
- * If the input is an {@link IContainer}, the tree is similar to the tree shown
- * by the PackageExplorer (based on {@link WorkbenchLabelProvider} and
- * {@link WorkbenchContentProvider}, otherwise a simple tree representing files
- * and folders is used based on {@link PathNode} instances. Linked resources
+ * In both cases the tree consists of {@link PathNode} instances representing
+ * files and folders. The container nodes are decorated with
+ * {@link WorkbenchLabelProvider} to use the same icons as the workbench. Linked resources
  * however are ignored and not listed as content.
  * <p>
  * The tree nodes are shown with icons for "Added", "Deleted", and
@@ -153,19 +147,9 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 
 	private IWorkbenchAction showEqualsAction;
 
-	private Map<IPath, GitFileRevision> compareVersionMap = new HashMap<IPath, GitFileRevision>();
+	private Map<IPath, FileNode> fileNodes = new HashMap<IPath, FileNode>();
 
-	private Map<IPath, GitFileRevision> baseVersionMap = new HashMap<IPath, GitFileRevision>();
-
-	private Set<IPath> addedPaths = new HashSet<IPath>();
-
-	private Set<IPath> equalContentPaths = new HashSet<IPath>();
-
-	private Set<IPath> baseVersionPathsWithChildren = new HashSet<IPath>();
-
-	private Map<IPath, List<PathNodeAdapter>> compareVersionPathsWithChildren = new HashMap<IPath, List<PathNodeAdapter>>();
-
-	private Set<IPath> deletedPaths = new HashSet<IPath>();
+	private Map<IPath, ContainerNode> containerNodes = new HashMap<IPath, ContainerNode>();
 
 	private List<IWorkbenchAction> actionsToDispose = new ArrayList<IWorkbenchAction>();
 
@@ -246,102 +230,39 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 				.getFirstElement();
 		ITypedElement left;
 		ITypedElement right;
-		if (selected instanceof IContainer) {
+		if (selected instanceof ContainerNode) {
 			// open/close folder
 			TreeViewer tv = (TreeViewer) event.getViewer();
 			tv.setExpandedState(selected, !tv.getExpandedState(selected));
-			return;
-		} else if (selected instanceof IFile) {
-			final IFile res = (IFile) selected;
-			LocalFileRevision revision = new LocalFileRevision(res);
-			left = new FileEditableRevision(revision, res, PlatformUI.getWorkbench().getProgressService());
-			GitFileRevision rightRevision = compareVersionMap.get(new Path(
-					repositoryMapping.getRepoRelativePath(res)));
-			if (rightRevision == null) {
-				right = new GitCompareFileRevisionEditorInput.EmptyTypedElement(
-						NLS
-								.bind(
-										UIText.CompareTreeView_ItemNotFoundInVersionMessage,
-										res.getName(), getCompareVersion()));
-			} else {
-				String encoding = CompareCoreUtils.getResourceEncoding(res);
-				right = new FileRevisionTypedElement(rightRevision, encoding);
-			}
-		} else if (selected instanceof GitFileRevision) {
-			GitFileRevision rightRevision = (GitFileRevision) selected;
-			left = new GitCompareFileRevisionEditorInput.EmptyTypedElement(NLS
-					.bind(UIText.CompareTreeView_ItemNotFoundInVersionMessage,
-							rightRevision.getName(), getBaseVersion()));
-			right = new FileRevisionTypedElement(rightRevision, null);
-		} else if (selected instanceof PathNode) {
-			PathNode node = (PathNode) selected;
-			String encoding = CompareCoreUtils.getResourceEncoding(
-					getRepository(), node.getRepoRelativePath());
-			switch (node.type) {
-			case FILE_BOTH_SIDES_DIFFER:
-				// fall through
-			case FILE_BOTH_SIDES_SAME: {
-				// open a compare editor with both sides filled
-				GitFileRevision rightRevision = compareVersionMap
-						.get(node.path);
-				right = new FileRevisionTypedElement(rightRevision, encoding);
-				GitFileRevision leftRevision = baseVersionMap.get(node.path);
-				left = new FileRevisionTypedElement(leftRevision, encoding);
-				break;
-			}
-			case FILE_DELETED: {
-				// open compare editor with left side empty
-				GitFileRevision rightRevision = compareVersionMap
-						.get(node.path);
-				right = new FileRevisionTypedElement(rightRevision, encoding);
-				left = new GitCompareFileRevisionEditorInput.EmptyTypedElement(
-						NLS
-								.bind(
-										UIText.CompareTreeView_ItemNotFoundInVersionMessage,
-										rightRevision.getName(),
-										getBaseVersion()));
-				break;
-			}
-			case FILE_ADDED: {
-				// open compare editor with right side empty
-				GitFileRevision leftRevision = baseVersionMap.get(node.path);
-				left = new FileRevisionTypedElement(leftRevision, encoding);
-				right = new GitCompareFileRevisionEditorInput.EmptyTypedElement(
-						NLS
-								.bind(
-										UIText.CompareTreeView_ItemNotFoundInVersionMessage,
-										leftRevision.getName(),
-										getCompareVersion()));
-				break;
-			}
-			case FOLDER:
-				// open/close folder
-				TreeViewer tv = (TreeViewer) event.getViewer();
-				tv.setExpandedState(selected, !tv.getExpandedState(selected));
-				return;
-			default:
-				return;
-			}
+		} else if (selected instanceof FileNode) {
+			FileNode fileNode = (FileNode) selected;
+			left = getTypedElement(fileNode, fileNode.leftRevision, getBaseVersion());
+			right = getTypedElement(fileNode, fileNode.rightRevision, getCompareVersion());
 
-		} else if (selected instanceof PathNodeAdapter) {
-			// deleted in workspace
-			PathNodeAdapter node = (PathNodeAdapter) selected;
-			GitFileRevision rightRevision = compareVersionMap
-					.get(node.pathNode.path);
-			String encoding = CompareCoreUtils.getResourceEncoding(
-					getRepository(), node.pathNode.getRepoRelativePath());
-			right = new FileRevisionTypedElement(rightRevision, encoding);
-			left = new GitCompareFileRevisionEditorInput.EmptyTypedElement(NLS
-					.bind(UIText.CompareTreeView_ItemNotFoundInVersionMessage,
-							node.pathNode.path.lastSegment(), getBaseVersion()));
-		} else
-			return;
+			GitCompareFileRevisionEditorInput compareInput = new GitCompareFileRevisionEditorInput(
+					left, right, PlatformUI.getWorkbench()
+							.getActiveWorkbenchWindow().getActivePage());
+			CompareUtils.openInCompare(PlatformUI.getWorkbench()
+					.getActiveWorkbenchWindow().getActivePage(), compareInput);
+		}
+	}
 
-		GitCompareFileRevisionEditorInput compareInput = new GitCompareFileRevisionEditorInput(
-				left, right, PlatformUI.getWorkbench()
-						.getActiveWorkbenchWindow().getActivePage());
-		CompareUtils.openInCompare(PlatformUI.getWorkbench()
-				.getActiveWorkbenchWindow().getActivePage(), compareInput);
+	private ITypedElement getTypedElement(FileNode node, IFileRevision fileRevision, String versionName) {
+		if (fileRevision instanceof LocalFileRevision) {
+			LocalFileRevision localFileRevision = (LocalFileRevision) fileRevision;
+			return new ResourceEditableRevision(fileRevision, localFileRevision.getFile(), PlatformUI.getWorkbench().getProgressService());
+		} else if (fileRevision == null) {
+			return new GitCompareFileRevisionEditorInput.EmptyTypedElement(
+					NLS.bind(
+							UIText.CompareTreeView_ItemNotFoundInVersionMessage,
+							node.getPath(), versionName));
+		} else {
+			Repository repository = getRepository();
+			String encoding = repository == null ? null
+					: CompareCoreUtils.getResourceEncoding(repository,
+							node.getRepoRelativePath());
+			return new FileRevisionTypedElement(fileRevision, encoding);
+		}
 	}
 
 	private String getBaseVersion() {
@@ -352,7 +273,10 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 	}
 
 	private String getCompareVersion() {
-		return compareVersion;
+		if (compareVersion.equals(INDEX_VERSION))
+			return UIText.CompareTreeView_IndexVersionText;
+		else
+			return compareVersion;
 	}
 
 	@Override
@@ -467,9 +391,7 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 				setContentDescription(NLS
 						.bind(
 								UIText.CompareTreeView_ComparingWorkspaceVersionDescription,
-								name,
-								compareVersion.equals(INDEX_VERSION) ? UIText.CompareTreeView_IndexVersionText
-										: compareVersion));
+								name, getCompareVersion()));
 			else
 				setContentDescription(NLS
 						.bind(
@@ -477,8 +399,7 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 								new String[] {
 										baseVersion,
 										name,
-										compareVersion.equals(INDEX_VERSION) ? UIText.CompareTreeView_IndexVersionText
-												: compareVersion }));
+										getCompareVersion() }));
 		}
 	}
 
@@ -488,15 +409,11 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 
 		tree.setInput(null);
 
-		if (baseVersion == null) {
-			tree.setContentProvider(new WorkbenchTreeContentProvider());
-			tree.setComparator(new WorkbenchTreeComparator());
-			tree.setLabelProvider(new WorkbenchTreeLabelProvider());
-		} else {
-			tree.setContentProvider(new PathNodeContentProvider());
-			tree.setComparator(new PathNodeTreeComparator());
-			tree.setLabelProvider(new PathNodeLabelProvider());
-		}
+		tree.setContentProvider(new PathNodeContentProvider());
+		tree.setComparator(new PathNodeTreeComparator());
+		tree.setLabelProvider(new PathNodeLabelProvider());
+		tree.setFilters(new PathNodeFilter[] { new PathNodeFilter() });
+
 		for (IWorkbenchAction action : actionsToDispose)
 			action.setEnabled(false);
 
@@ -575,13 +492,8 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 		monitor.beginTask(UIText.CompareTreeView_AnalyzingRepositoryTaskText,
 				IProgressMonitor.UNKNOWN);
 		boolean useIndex = compareVersion.equals(INDEX_VERSION);
-		deletedPaths.clear();
-		equalContentPaths.clear();
-		baseVersionMap.clear();
-		compareVersionMap.clear();
-		compareVersionPathsWithChildren.clear();
-		addedPaths.clear();
-		baseVersionPathsWithChildren.clear();
+		fileNodes.clear();
+		containerNodes.clear();
 		boolean checkIgnored = false;
 		TreeWalk tw = new TreeWalk(repository);
 		try {
@@ -641,93 +553,77 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 						compareTreeIndex, AbstractTreeIterator.class);
 				AbstractTreeIterator baseVersionIterator = tw.getTree(
 						baseTreeIndex, AbstractTreeIterator.class);
-				if (compareVersionIterator != null
-						&& baseVersionIterator != null) {
-					monitor.setTaskName(baseVersionIterator
-							.getEntryPathString());
-					IPath currentPath = new Path(baseVersionIterator
-							.getEntryPathString());
-					if (!useIndex)
-						compareVersionMap
-								.put(currentPath, GitFileRevision.inCommit(
-										repository, compareCommit,
-										baseVersionIterator
-												.getEntryPathString(), tw
-												.getObjectId(compareTreeIndex)));
-					else
-						compareVersionMap.put(currentPath, GitFileRevision
-								.inIndex(repository, baseVersionIterator
-										.getEntryPathString()));
-					if (baseCommit != null)
-						baseVersionMap.put(currentPath, GitFileRevision
+
+				IFileRevision left = null;
+				IFileRevision right = null;
+				String repoRelativePath = baseVersionIterator != null
+						? baseVersionIterator.getEntryPathString()
+						: compareVersionIterator.getEntryPathString();
+				IPath currentPath = new Path(repoRelativePath);
+
+				monitor.setTaskName(currentPath.toString());
+
+				IFile file = ResourceUtil.getFileForLocation(repository,
+						repoRelativePath);
+				if (baseVersionIterator != null) {
+					if (baseCommit == null) {
+						if (file != null)
+							left = new LocalFileRevision(file);
+						else {
+							IPath path = getRepositoryPath().append(
+									repoRelativePath);
+							left = new WorkingTreeFileRevision(
+									path.toFile());
+						}
+					} else {
+						left = GitFileRevision
 								.inCommit(repository, baseCommit,
-										baseVersionIterator
-												.getEntryPathString(), tw
-												.getObjectId(baseTreeIndex)));
+										repoRelativePath,
+										tw.getObjectId(baseTreeIndex));
+					}
+				}
+
+				if (compareVersionIterator != null) {
+					if (!useIndex)
+						right = GitFileRevision.inCommit(repository,
+								compareCommit, repoRelativePath,
+								tw.getObjectId(compareTreeIndex));
+					else
+						right = GitFileRevision.inIndex(repository,
+								repoRelativePath);
+				}
+
+				Type type = null;
+				if (compareVersionIterator != null && baseVersionIterator != null) {
 					boolean equalContent = compareVersionIterator
 							.getEntryObjectId().equals(
 									baseVersionIterator.getEntryObjectId());
-					if (equalContent)
-						equalContentPaths.add(currentPath);
+					type = equalContent ? Type.FILE_BOTH_SIDES_SAME : Type.FILE_BOTH_SIDES_DIFFER;
+				} else if (compareVersionIterator != null && baseVersionIterator == null) {
+					type = Type.FILE_DELETED;
+				} else if (compareVersionIterator == null && baseVersionIterator != null) {
+					type = Type.FILE_ADDED;
+				}
 
-					if (equalContent && !showEquals)
-						continue;
+				IPath containerPath = currentPath.removeLastSegments(1);
+				ContainerNode containerNode = getOrCreateContainerNode(
+						containerPath, type);
 
-					while (currentPath.segmentCount() > 0) {
-						currentPath = currentPath.removeLastSegments(1);
-						if (!baseVersionPathsWithChildren.add(currentPath))
-							break;
+				FileNode fileNode = new FileNode(currentPath, file, type, left,
+						right);
+				containerNode.addChild(fileNode);
+				fileNodes.put(currentPath, fileNode);
+
+				// If a file is not "equal content", the container nodes up to
+				// the root must be shown in any case, so propagate the
+				// change of the "only equal content" flag.
+				if (type != Type.FILE_BOTH_SIDES_SAME) {
+					IPath path = currentPath;
+					while (path.segmentCount() > 0) {
+						path = path.removeLastSegments(1);
+						ContainerNode node = containerNodes.get(path);
+						node.setOnlyEqualContent(false);
 					}
-
-				} else if (baseVersionIterator != null
-						&& compareVersionIterator == null) {
-					monitor.setTaskName(baseVersionIterator
-							.getEntryPathString());
-					// only on base side
-					IPath currentPath = new Path(baseVersionIterator
-							.getEntryPathString());
-					addedPaths.add(currentPath);
-					if (baseCommit != null)
-						baseVersionMap.put(currentPath, GitFileRevision
-								.inCommit(repository, baseCommit,
-										baseVersionIterator
-												.getEntryPathString(), tw
-												.getObjectId(baseTreeIndex)));
-					while (currentPath.segmentCount() > 0) {
-						currentPath = currentPath.removeLastSegments(1);
-						if (!baseVersionPathsWithChildren.add(currentPath))
-							break;
-					}
-
-				} else if (compareVersionIterator != null
-						&& baseVersionIterator == null) {
-					monitor.setTaskName(compareVersionIterator
-							.getEntryPathString());
-					// only on compare side
-					IPath currentPath = new Path(compareVersionIterator
-							.getEntryPathString());
-					deletedPaths.add(currentPath);
-					List<PathNodeAdapter> children = compareVersionPathsWithChildren
-							.get(currentPath.removeLastSegments(1));
-					if (children == null) {
-						children = new ArrayList<PathNodeAdapter>(1);
-						compareVersionPathsWithChildren.put(currentPath
-								.removeLastSegments(1), children);
-					}
-					children.add(new PathNodeAdapter(new PathNode(currentPath,
-							Type.FILE_DELETED)));
-
-					if (!useIndex)
-						compareVersionMap
-								.put(currentPath, GitFileRevision.inCommit(
-										repository, compareCommit,
-										compareVersionIterator
-												.getEntryPathString(), tw
-												.getObjectId(compareTreeIndex)));
-					else
-						compareVersionMap.put(currentPath, GitFileRevision
-								.inIndex(repository, compareVersionIterator
-										.getEntryPathString()));
 				}
 			}
 		} finally {
@@ -736,6 +632,28 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 		}
 	}
 
+	private ContainerNode getOrCreateContainerNode(IPath containerPath, Type fileType) {
+		ContainerNode containerNode = containerNodes.get(containerPath);
+		if (containerNode != null) {
+			return containerNode;
+		} else {
+			Repository repository = getRepository();
+			IContainer resource = repository == null ? null : ResourceUtil
+					.getContainerForLocation(repository,
+							containerPath.toString());
+			ContainerNode node = new ContainerNode(containerPath, resource);
+			node.setOnlyEqualContent(fileType == Type.FILE_BOTH_SIDES_SAME);
+			containerNodes.put(containerPath, node);
+			if (containerPath.segmentCount() > 0) {
+				IPath parentPath = containerPath.removeLastSegments(1);
+				ContainerNode parentNode = getOrCreateContainerNode(parentPath,
+						fileType);
+				parentNode.addChild(node);
+			}
+			return node;
+		}
+	}
+
 	@Override
 	public void dispose() {
 		super.dispose();
@@ -746,7 +664,10 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 		SAME_CONTENT.dispose();
 	}
 
-	final static class PathNode {
+	/**
+	 * Base class of tree nodes.
+	 */
+	static abstract class PathNode {
 		/** Type; note that the ordinal is used to sort the tree */
 		public enum Type {
 			/** Folder */
@@ -761,17 +682,19 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 			FILE_BOTH_SIDES_SAME
 		}
 
-		public final IPath path;
-
-		public final Type type;
+		private final IPath path;
 
 		/**
 		 * @param path
-		 * @param type
 		 */
-		public PathNode(IPath path, Type type) {
+		public PathNode(IPath path) {
 			this.path = path;
-			this.type = type;
+		}
+
+		public abstract Type getType();
+
+		public IPath getPath() {
+			return path;
 		}
 
 		public String getRepoRelativePath() {
@@ -783,7 +706,7 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 			final int prime = 31;
 			int result = 1;
 			result = prime * result + path.hashCode();
-			result = prime * result + type.hashCode();
+			result = prime * result + getType().hashCode();
 			return result;
 		}
 
@@ -798,112 +721,96 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 			PathNode other = (PathNode) obj;
 			if (!path.equals(other.path))
 				return false;
-			if (!type.equals(other.type))
+			if (!getType().equals(other.getType()))
 				return false;
 			return true;
 		}
 
 		@Override
 		public String toString() {
-			return type.name() + ": " + path.toString(); //$NON-NLS-1$
-		}
-	}
-
-	private final class WorkbenchTreeLabelProvider extends
-			WorkbenchLabelProvider {
-		@Override
-		protected ImageDescriptor decorateImage(ImageDescriptor baseImage,
-				Object element) {
-			if (!(element instanceof IFile)) {
-				return super.decorateImage(baseImage, element);
-			}
-			IPath elementPath = new Path(repositoryMapping
-					.getRepoRelativePath((IFile) element));
-			// decorate with + for files not found in the repository and = for
-			// "same" files
-			if (addedPaths.contains(elementPath)) {
-				return UIIcons.ELCL16_ADD;
-			}
-			if (deletedPaths.contains(elementPath)) {
-				return UIIcons.ELCL16_DELETE;
-			}
-			if (equalContentPaths.contains(elementPath)) {
-				ImageDescriptor imageDesc = super.decorateImage(baseImage,
-						element);
-				return ImageDescriptor.createWithFlags(imageDesc,
-						SWT.IMAGE_DISABLE);
-			}
-			return super.decorateImage(baseImage, element);
+			return getType().name() + ": " + path.toString(); //$NON-NLS-1$
 		}
 	}
 
 	/**
-	 * Sorts the workbench tree by type and state (Folder, Added, Deleted,
-	 * Changed, Unchanged Files)
+	 * Subclass for container (folder) tree nodes.
 	 */
-	private final class WorkbenchTreeComparator extends ViewerComparator {
-		private static final int FOLDERCATEGORY = 5;
+	static class ContainerNode extends PathNode {
 
-		private static final int ADDEDCATEGORY = 10;
+		private final List<PathNode> children = new ArrayList<PathNode>();
+		private final IContainer resource;
+		private boolean onlyEqualContent = false;
 
-		private static final int DELETEDCATEGORY = 15;
-
-		private static final int CHANGEDCATEGORY = 20;
-
-		private static final int UNCHANGEDCATEGORY = 30;
-
-		private static final int UNKNOWNCATEGORY = 50;
-
-		@Override
-		public int category(Object element) {
-			IResource adapter = (IResource) getAdapter(element, IResource.class);
-			if (adapter != null) {
-				if (adapter instanceof IContainer) {
-					return FOLDERCATEGORY;
-				}
-				if (adapter instanceof IFile) {
-					IFile file = (IFile) adapter;
-					IPath path = new Path(repositoryMapping
-							.getRepoRelativePath(file));
-					if (addedPaths.contains(path))
-						return ADDEDCATEGORY;
-					if (equalContentPaths.contains(path))
-						return UNCHANGEDCATEGORY;
-					return CHANGEDCATEGORY;
-				}
-			}
-			if (element instanceof PathNodeAdapter)
-				return DELETEDCATEGORY;
-
-			return UNKNOWNCATEGORY;
+		/**
+		 * @param path
+		 * @param resource container resource, may be null
+		 */
+		public ContainerNode(IPath path, IContainer resource) {
+			super(path);
+			this.resource = resource;
 		}
 
-		private Object getAdapter(Object sourceObject, Class adapterType) {
-			Assert.isNotNull(adapterType);
-			if (sourceObject == null)
-				return null;
+		@Override
+		public Type getType() {
+			return Type.FOLDER;
+		}
 
-			if (adapterType.isInstance(sourceObject))
-				return sourceObject;
+		public List<PathNode> getChildren() {
+			return children;
+		}
 
-			if (sourceObject instanceof IAdaptable) {
-				IAdaptable adaptable = (IAdaptable) sourceObject;
+		public void addChild(PathNode child) {
+			children.add(child);
+		}
 
-				Object result = adaptable.getAdapter(adapterType);
-				if (result != null) {
-					// Sanity-check
-					Assert.isTrue(adapterType.isInstance(result));
-					return result;
-				}
-			}
+		public IContainer getResource() {
+			return resource;
+		}
 
-			if (!(sourceObject instanceof PlatformObject)) {
-				Object result = Platform.getAdapterManager().getAdapter(
-						sourceObject, adapterType);
-				if (result != null)
-					return result;
-			}
-			return null;
+		public void setOnlyEqualContent(boolean onlyEqualContent) {
+			this.onlyEqualContent = onlyEqualContent;
+		}
+
+		public boolean isOnlyEqualContent() {
+			return onlyEqualContent;
+		}
+	}
+
+	/**
+	 * Subclass for file tree nodes.
+	 */
+	static class FileNode extends PathNode {
+
+		private final IFile file;
+		private final Type type;
+		private final IFileRevision leftRevision;
+		private final IFileRevision rightRevision;
+
+		/**
+		 * @param path
+		 * @param file
+		 * @param type
+		 * @param leftRevision
+		 *            left revision, may be null
+		 * @param rightRevision
+		 *            right revision, may be null
+		 */
+		public FileNode(IPath path, IFile file, Type type,
+				IFileRevision leftRevision, IFileRevision rightRevision) {
+			super(path);
+			this.file = file;
+			this.type = type;
+			this.leftRevision = leftRevision;
+			this.rightRevision = rightRevision;
+		}
+
+		@Override
+		public Type getType() {
+			return type;
+		}
+
+		public IFile getFile() {
+			return file;
 		}
 	}
 
@@ -917,166 +824,73 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 		@Override
 		public int category(Object element) {
 			if (element instanceof PathNode) {
-				return ((PathNode) element).type.ordinal();
+				return ((PathNode) element).getType().ordinal();
 			}
 			return UNKNOWNCATEGORY;
 		}
 	}
 
 	/**
-	 * Used to render the "local" (workspace) side of a tree compare where one
-	 * side is the workspace
+	 * Content provider working with {@link PathNode} elements.
 	 */
-	private final class WorkbenchTreeContentProvider extends
-			WorkbenchContentProvider {
-		@Override
-		public Object[] getChildren(Object element) {
-			boolean rebuildArray = false;
-			Object[] children;
-			if (element == input)
-				children = (Object[]) input;
-			else
-				children = super.getChildren(element);
-			List<Object> childList = new ArrayList<Object>(children.length);
-			for (Object child : children) {
-				IResource childResource = (IResource) child;
-				if (childResource.isLinked())
-					continue;
-				IPath path = new Path(
-						repositoryMapping.getRepoRelativePath(childResource));
-				boolean isFile = childResource.getType() == IResource.FILE;
+	private final class PathNodeContentProvider implements ITreeContentProvider {
 
-				// each path that is not ignored creates an entry in either
-				// compareVersionMap or addedPaths, so we can check if a path
-				// was ignored by looking into these tables
-				if (isFile && !compareVersionMap.containsKey(path)
-						&& !addedPaths.contains(path)) {
-					rebuildArray = true;
-					continue;
-				}
-				if (!showEquals && equalContentPaths.contains(path)) {
-					rebuildArray = true;
-					continue;
-				}
-				if (child instanceof IContainer
-						&& !baseVersionPathsWithChildren.contains(path)) {
-					rebuildArray = true;
-					continue;
-				}
-				if (!showEquals && equalContentPaths.contains(path)) {
-					rebuildArray = true;
-					continue;
-				}
-				childList.add(child);
-			}
-			if (element instanceof IContainer) {
-				List<PathNodeAdapter> deletedChildren = compareVersionPathsWithChildren
-						.get(new Path(repositoryMapping
-								.getRepoRelativePath((IResource) element)));
-				if (deletedChildren != null) {
-					rebuildArray = true;
-					for (IWorkbenchAdapter path : deletedChildren)
-						childList.add(path);
-				}
-			}
-			if (rebuildArray)
-				return childList.toArray();
-			return children;
-		}
-	}
-
-	/**
-	 * Used to render the tree in case we have no workspace
-	 */
-	private final class PathNodeContentProvider extends ArrayContentProvider
-			implements ITreeContentProvider {
-		@Override
 		public Object[] getElements(Object inputElement) {
-			if (baseVersionPathsWithChildren.isEmpty() && addedPaths.isEmpty())
+			ContainerNode rootContainer = containerNodes.get(new Path("")); //$NON-NLS-1$
+			if (rootContainer.isOnlyEqualContent() && !showEquals)
 				return new String[] { UIText.CompareTreeView_NoDifferencesFoundMessage };
+
 			if (input instanceof IResource[]) {
 				IResource[] resources = (IResource[]) input;
 				PathNode[] nodes = new PathNode[resources.length];
 				for (int i = 0; i < resources.length; i++) {
 					IResource resource = resources[i];
-					if (resource instanceof IFile) {
-						IPath path = new Path(repositoryMapping
-								.getRepoRelativePath(resource));
-						Type type;
-						if (addedPaths.contains(path))
-							type = Type.FILE_ADDED;
-						else if (equalContentPaths.contains(path))
-							type = Type.FILE_BOTH_SIDES_SAME;
-						else
-							type = Type.FILE_BOTH_SIDES_DIFFER;
-						nodes[i] = new PathNode(path, type);
-					} else
-						nodes[i] = new PathNode(new Path(repositoryMapping
-								.getRepoRelativePath(resource)), Type.FOLDER);
+					IPath path = new Path(repositoryMapping
+							.getRepoRelativePath(resource));
+					if (resource instanceof IFile)
+						nodes[i] = fileNodes.get(path);
+					else
+						nodes[i] = containerNodes.get(path);
 				}
 				return nodes;
-			}
-			return new PathNode[] { new PathNode(new Path(""), Type.FOLDER) }; //$NON-NLS-1$
+			} else
+				return new PathNode[] { rootContainer };
 		}
 
 		public Object[] getChildren(Object parentElement) {
-			PathNode parentNode = (PathNode) parentElement;
-			IPath parent = parentNode.path;
-			List<PathNode> children = new ArrayList<PathNode>();
-			for (IPath childPath : baseVersionPathsWithChildren) {
-				if (childPath.segmentCount() > 0
-						&& childPath.removeLastSegments(1).equals(parent)) {
-					children.add(new PathNode(childPath, Type.FOLDER));
-				}
-			}
-			for (IPath mapPath : baseVersionMap.keySet()) {
-				if (mapPath.removeLastSegments(1).equals(parent)
-						&& (showEquals || !equalContentPaths.contains(mapPath))) {
-					if (addedPaths.contains(mapPath))
-						children.add(new PathNode(mapPath, Type.FILE_ADDED));
-					else if (equalContentPaths.contains(mapPath))
-						children.add(new PathNode(mapPath,
-								Type.FILE_BOTH_SIDES_SAME));
-					else
-						children.add(new PathNode(mapPath,
-								Type.FILE_BOTH_SIDES_DIFFER));
-				}
-			}
-			if (parentNode.type == Type.FOLDER) {
-				List<PathNodeAdapter> deletedChildren = compareVersionPathsWithChildren
-						.get(parent);
-				if (deletedChildren != null)
-					for (PathNodeAdapter path : deletedChildren)
-						children.add(path.pathNode);
-
-			}
-			return children.toArray();
+			if (parentElement instanceof ContainerNode) {
+				ContainerNode containerNode = (ContainerNode) parentElement;
+				return containerNode.getChildren().toArray();
+			} else
+				return new Object[] {};
 		}
 
 		public boolean hasChildren(Object element) {
-			if (!(element instanceof PathNode))
+			if (element instanceof ContainerNode) {
+				ContainerNode containerNode = (ContainerNode) element;
+				return !containerNode.getChildren().isEmpty();
+			} else
 				return false;
-			IPath parent = ((PathNode) element).path;
-			for (IPath childPath : baseVersionPathsWithChildren) {
-				if (childPath.removeLastSegments(1).equals(parent))
-					return true;
-			}
-			for (IPath mapPath : baseVersionMap.keySet()) {
-				if (mapPath.removeLastSegments(1).equals(parent)
-						&& (showEquals || !equalContentPaths.contains(mapPath))) {
-					return true;
-				}
-			}
-			return false;
 		}
 
 		public Object getParent(Object element) {
-			if (!(element instanceof PathNode))
+			if (element instanceof PathNode) {
+				PathNode pathNode = (PathNode) element;
+				IPath path = pathNode.getPath();
+				if (path.segmentCount() == 0)
+					return null;
+				else
+					return containerNodes.get(path.removeLastSegments(1));
+			} else
 				return null;
-			IPath currentPath = ((PathNode) element).path;
-			if (currentPath.segmentCount() > 0)
-				return currentPath.removeLastSegments(1);
-			return null;
+		}
+
+		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
+			// Do nothing
+		}
+
+		public void dispose() {
+			// Do nothing
 		}
 	}
 
@@ -1085,15 +899,30 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 	 */
 	private final class PathNodeLabelProvider extends BaseLabelProvider
 			implements ILabelProvider {
+
+		private final WorkbenchLabelProvider workbenchLabelProvider = new WorkbenchLabelProvider();
+
 		public Image getImage(Object element) {
 			if (element instanceof String)
 				return null;
-			Type type = ((PathNode) element).type;
+			if (element instanceof ContainerNode) {
+				ContainerNode containerNode = (ContainerNode) element;
+				IContainer resource = containerNode.getResource();
+				if (resource != null && resource.isAccessible())
+					return workbenchLabelProvider.getImage(resource);
+				else
+					return FOLDER_IMAGE;
+			}
+			FileNode fileNode = (FileNode) element;
+			Type type = fileNode.getType();
 			switch (type) {
 			case FILE_BOTH_SIDES_SAME:
 				return SAME_CONTENT;
 			case FILE_BOTH_SIDES_DIFFER:
-				return FILE_IMAGE;
+				if (fileNode.getFile() != null)
+					return workbenchLabelProvider.getImage(fileNode.getFile());
+				else
+					return FILE_IMAGE;
 			case FILE_ADDED:
 				return ADDED;
 			case FILE_DELETED:
@@ -1107,38 +936,39 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 		public String getText(Object element) {
 			if (element instanceof String)
 				return (String) element;
-			IPath path = ((PathNode) element).path;
+			IPath path = ((PathNode) element).getPath();
 			if (path.segmentCount() == 0)
 				return UIText.CompareTreeView_RepositoryRootName;
 			return path.lastSegment();
+		}
 
+		@Override
+		public void dispose() {
+			super.dispose();
+			workbenchLabelProvider.dispose();
 		}
 	}
 
-	private final static class PathNodeAdapter implements IWorkbenchAdapter {
-		private final static Object[] EMPTYARRAY = new Object[0];
-
-		PathNode pathNode;
-
-		public PathNodeAdapter(PathNode path) {
-			pathNode = path;
-		}
-
-		public Object[] getChildren(Object o) {
-			return EMPTYARRAY;
-		}
-
-		public ImageDescriptor getImageDescriptor(Object object) {
-			return UIIcons.ELCL16_DELETE;
-		}
-
-		public String getLabel(Object o) {
-			return pathNode.path.lastSegment();
-		}
-
-		public Object getParent(Object o) {
-			// doesn't seem to hurt to simply return null
-			return null;
+	/**
+	 * Filter out equal path nodes if "show equals" is disabled.
+	 */
+	private final class PathNodeFilter extends ViewerFilter {
+		@Override
+		public boolean select(Viewer viewer, Object parentElement,
+				Object element) {
+			if (showEquals)
+				return true;
+			else {
+				if (element instanceof FileNode) {
+					FileNode fileNode = (FileNode) element;
+					return fileNode.getType() != Type.FILE_BOTH_SIDES_SAME;
+				} else if (element instanceof ContainerNode) {
+					ContainerNode containerNode = (ContainerNode) element;
+					return !containerNode.isOnlyEqualContent();
+				} else if (element instanceof String)
+					return true;
+			}
+			return true;
 		}
 	}
 
@@ -1233,10 +1063,10 @@ public class CompareTreeView extends ViewPart implements IMenuListener, IShowInS
 				pathsToOpen.add(((IFile) obj).getLocation().toOSString());
 			} else if (obj instanceof PathNode && repoPath != null) {
 				PathNode pathNode = (PathNode) obj;
-				if (pathNode.type == PathNode.Type.FOLDER
-						|| pathNode.type == PathNode.Type.FILE_DELETED)
+				if (pathNode.getType() == PathNode.Type.FOLDER
+						|| pathNode.getType() == PathNode.Type.FILE_DELETED)
 					return null;
-				pathsToOpen.add(repoPath.append(pathNode.path).toOSString());
+				pathsToOpen.add(repoPath.append(pathNode.getPath()).toOSString());
 			} else {
 				return null; // fail if one of the selected elements does not fit
 			}
