@@ -1,8 +1,5 @@
 /*******************************************************************************
- * Copyright (C) 2010, Benjamin Muskalla <bmuskalla@eclipsesource.com>
- * Copyright (C) 2011, Matthias Sohn <matthias.sohn@sap.com>
- * Copyright (C) 2011-2012, IBM Corporation
- * Copyright (C) 2012, Robin Stocker <robin@nibor.org>
+ * Copyright (C) 2010, 2013 Benjamin Muskalla <bmuskalla@eclipsesource.com> and others.
  *
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -195,7 +192,7 @@ public class SpellcheckableMessageArea extends Composite {
 			}
 		}
 
-		private String getPreferenceKey() {
+		protected String getPreferenceKey() {
 			return preferenceKey;
 		}
 
@@ -220,6 +217,10 @@ public class SpellcheckableMessageArea extends Composite {
 		protected ITextViewer getTextViewer() {
 			return viewer;
 		}
+
+		protected IPreferenceStore getStore() {
+			return store;
+		}
 	}
 
 	private final SourceViewer sourceViewer;
@@ -268,6 +269,7 @@ public class SpellcheckableMessageArea extends Composite {
 		AnnotationModel annotationModel = new AnnotationModel();
 		sourceViewer = new SourceViewer(this, null, null, true, SWT.MULTI
 				| SWT.V_SCROLL | SWT.WRAP);
+		getTextWidget().setAlwaysShowScrollBars(false);
 		getTextWidget().setFont(UIUtils
 				.getFont(UIPreferences.THEME_CommitMessageEditorFont));
 
@@ -510,6 +512,38 @@ public class SpellcheckableMessageArea extends Composite {
 
 			private IPainter whitespaceCharPainter;
 
+			@Override
+			public void propertyChange(PropertyChangeEvent event) {
+				String property = event.getProperty();
+				if (property.equals(getPreferenceKey())
+						|| AbstractTextEditor.PREFERENCE_SHOW_LEADING_SPACES
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_ENCLOSED_SPACES
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_TRAILING_SPACES
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_LEADING_IDEOGRAPHIC_SPACES
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_ENCLOSED_IDEOGRAPHIC_SPACES
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_TRAILING_IDEOGRAPHIC_SPACES
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_LEADING_TABS
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_ENCLOSED_TABS
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_TRAILING_TABS
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_CARRIAGE_RETURN
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_SHOW_LINE_FEED
+								.equals(property)
+						|| AbstractTextEditor.PREFERENCE_WHITESPACE_CHARACTER_ALPHA_VALUE
+								.equals(property)) {
+					synchronizeWithPreference();
+				}
+			}
+
 			@Override
 			protected void toggleState(boolean checked) {
 				if (checked)
@@ -525,7 +559,21 @@ public class SpellcheckableMessageArea extends Composite {
 				Assert.isTrue(whitespaceCharPainter == null);
 				ITextViewer v = getTextViewer();
 				if (v instanceof ITextViewerExtension2) {
-					whitespaceCharPainter = new WhitespaceCharacterPainter(v);
+					IPreferenceStore store = getStore();
+					whitespaceCharPainter = new WhitespaceCharacterPainter(
+							v,
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_LEADING_SPACES),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_ENCLOSED_SPACES),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_TRAILING_SPACES),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_LEADING_IDEOGRAPHIC_SPACES),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_ENCLOSED_IDEOGRAPHIC_SPACES),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_TRAILING_IDEOGRAPHIC_SPACES),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_LEADING_TABS),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_ENCLOSED_TABS),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_TRAILING_TABS),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_CARRIAGE_RETURN),
+							store.getBoolean(AbstractTextEditor.PREFERENCE_SHOW_LINE_FEED),
+							store.getInt(AbstractTextEditor.PREFERENCE_WHITESPACE_CHARACTER_ALPHA_VALUE));
 					((ITextViewerExtension2) v).addPainter(whitespaceCharPainter);
 				}
 			}
