@@ -98,7 +98,8 @@ public class LocationPage extends WizardPage {
 
 	private IContainer wsSelectedContainer;
 
-	private class LocationPageContentProvider extends BaseWorkbenchContentProvider {
+	private static class LocationPageContentProvider extends
+			BaseWorkbenchContentProvider {
 		//Never show closed projects
 		boolean showClosedProjects = false;
 
