@@ -398,9 +398,15 @@ public class ConfigurationEditorComponent {
 						else
 							entry.addValue(dlg.getValue());
 						markDirty();
-					} else if (st.countTokens() == 3) {
+					} else if (st.countTokens() > 2) {
+						int n = st.countTokens();
 						String sectionName = st.nextToken();
-						String subSectionName = st.nextToken();
+						StringBuilder b = new StringBuilder(st.nextToken());
+						for (int i = 0; i < n - 3; i++) {
+							b.append(DOT);
+							b.append(st.nextToken());
+						}
+						String subSectionName = b.toString();
 						String entryName = st.nextToken();
 						Entry entry = ((GitConfig) tv.getInput()).getEntry(
 								sectionName, subSectionName, entryName);
