@@ -21,6 +21,7 @@ import java.util.Set;
 
 import org.eclipse.compare.CompareEditorInput;
 import org.eclipse.compare.CompareUI;
+import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
 import org.eclipse.core.resources.ResourcesPlugin;
@@ -28,18 +29,16 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IStatus;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.core.runtime.jobs.IJobChangeEvent;
-import org.eclipse.core.runtime.jobs.JobChangeAdapter;
 import org.eclipse.egit.core.internal.FileChecker;
 import org.eclipse.egit.core.internal.FileChecker.CheckResult;
 import org.eclipse.egit.core.internal.FileChecker.CheckResultEntry;
-import org.eclipse.egit.core.internal.job.JobUtil;
-import org.eclipse.egit.core.op.RebaseOperation;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
-import org.eclipse.egit.ui.JobFamilies;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.internal.UIText;
+import org.eclipse.egit.ui.internal.commands.shared.AbortRebaseCommand;
+import org.eclipse.egit.ui.internal.commands.shared.AbstractRebaseCommandHandler;
+import org.eclipse.egit.ui.internal.commands.shared.SkipRebaseCommand;
 import org.eclipse.egit.ui.internal.dialogs.CheckoutConflictDialog;
 import org.eclipse.egit.ui.internal.merge.GitMergeEditorInput;
 import org.eclipse.egit.ui.internal.merge.MergeModeDialog;
@@ -51,7 +50,6 @@ import org.eclipse.jface.viewers.ArrayContentProvider;
 import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.jface.viewers.TableViewer;
 import org.eclipse.jface.window.Window;
-import org.eclipse.jgit.api.RebaseCommand.Operation;
 import org.eclipse.jgit.api.RebaseResult;
 import org.eclipse.jgit.api.RebaseResult.Status;
 import org.eclipse.jgit.dircache.DirCache;
@@ -111,6 +109,7 @@ public class RebaseResultDialog extends MessageDialog {
 	public static void show(final RebaseResult result,
 			final Repository repository) {
 		boolean shouldShow = result.getStatus() == Status.STOPPED
+				|| result.getStatus() == Status.STASH_APPLY_CONFLICTS
 				|| Activator.getDefault().getPreferenceStore().getBoolean(
 						UIPreferences.SHOW_REBASE_CONFIRM);
 
@@ -151,6 +150,8 @@ public class RebaseResultDialog extends MessageDialog {
 			return UIText.RebaseResultDialog_Aborted;
 		case STOPPED:
 			return UIText.RebaseResultDialog_Stopped;
+		case EDIT:
+			return UIText.RebaseResultDialog_Edit;
 		case FAILED:
 			return UIText.RebaseResultDialog_Failed;
 		case UP_TO_DATE:
@@ -159,6 +160,13 @@ public class RebaseResultDialog extends MessageDialog {
 			return UIText.RebaseResultDialog_FastForward;
 		case NOTHING_TO_COMMIT:
 			return UIText.RebaseResultDialog_NothingToCommit;
+		case INTERACTIVE_PREPARED:
+			return UIText.RebaseResultDialog_InteractivePrepared;
+		case UNCOMMITTED_CHANGES:
+			return UIText.RebaseResultDialog_UncommittedChanges;
+		case STASH_APPLY_CONFLICTS:
+			return UIText.RebaseResultDialog_SuccessfullyFinished + ".\n" + //$NON-NLS-1$
+					UIText.RebaseResultDialog_stashApplyConflicts;
 		default:
 			throw new IllegalStateException(status.name());
 		}
@@ -186,6 +194,11 @@ public class RebaseResultDialog extends MessageDialog {
 			return UIText.RebaseResultDialog_StatusFastForward;
 		case NOTHING_TO_COMMIT:
 			return UIText.RebaseResultDialog_StatusNothingToCommit;
+		case UNCOMMITTED_CHANGES:
+			return UIText.RebaseResultDialog_UncommittedChanges;
+		case STASH_APPLY_CONFLICTS:
+			return UIText.RebaseResultDialog_SuccessfullyFinished + ".\n" + //$NON-NLS-1$
+					UIText.RebaseResultDialog_stashApplyConflicts;
 		}
 		return status.toString();
 	}
@@ -244,13 +257,13 @@ public class RebaseResultDialog extends MessageDialog {
 				false));
 		Text resultText = new Text(composite, SWT.READ_ONLY);
 		resultText.setText(getStatusText(result.getStatus()));
+		if (!result.getStatus().isSuccessful())
+			resultText.setForeground(composite.getParent().getDisplay()
+					.getSystemColor(SWT.COLOR_RED));
 		resultText.setSelection(resultText.getCaretPosition());
 		resultText.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
-		if (result.getStatus() == Status.FAILED) {
-			resultText.setForeground(composite.getParent().getDisplay()
-					.getSystemColor(
-					SWT.COLOR_RED));
 
+		if (result.getStatus() == Status.FAILED) {
 			StringBuilder paths = new StringBuilder();
 			Label pathsLabel = new Label(composite, SWT.NONE);
 			pathsLabel.setText(UIText.MergeResultDialog_failed);
@@ -280,9 +293,6 @@ public class RebaseResultDialog extends MessageDialog {
 			}
 			pathsText.setText(paths.toString());
 		} else if (result.getStatus() == Status.CONFLICTS) {
-			resultText.setForeground(composite.getParent().getDisplay()
-					.getSystemColor(SWT.COLOR_RED));
-
 			StringBuilder paths = new StringBuilder();
 			Label pathsLabel = new Label(composite, SWT.NONE);
 			pathsLabel.setText(UIText.MergeResultDialog_conflicts);
@@ -531,14 +541,12 @@ public class RebaseResultDialog extends MessageDialog {
 			if (startMergeButton.getSelection()) {
 				super.buttonPressed(buttonId);
 				// open the merge tool
-				List<IProject> validProjects = new ArrayList<IProject>();
 				IProject[] projects = ResourcesPlugin.getWorkspace().getRoot()
 						.getProjects();
 				for (IProject project : projects) {
 					RepositoryMapping mapping = RepositoryMapping
 							.getMapping(project);
 					if (mapping != null && mapping.getRepository().equals(repo)) {
-						validProjects.add(project);
 						try {
 							// make sure to refresh before opening the merge
 							// tool
@@ -550,20 +558,14 @@ public class RebaseResultDialog extends MessageDialog {
 						}
 					}
 				}
-				List<IResource> resourceList = new ArrayList<IResource>();
+				List<IPath> locationList = new ArrayList<IPath>();
 				IPath repoWorkdirPath = new Path(repo.getWorkTree().getPath());
 				for (String repoPath : conflictPaths) {
-					IPath filePath = repoWorkdirPath.append(repoPath);
-					for (IProject project : validProjects)
-						if (project.getLocation().isPrefixOf(filePath)) {
-							IResource res = project.getFile(filePath
-									.removeFirstSegments(project.getLocation()
-											.segmentCount()));
-							resourceList.add(res);
-						}
+					IPath location = repoWorkdirPath.append(repoPath);
+					locationList.add(location);
 				}
-				IResource[] resources = new IResource[resourceList.size()];
-				resourceList.toArray(resources);
+				IPath[] locations = locationList.toArray(new IPath[locationList
+						.size()]);
 				int mergeMode = Activator.getDefault().getPreferenceStore()
 						.getInt(UIPreferences.MERGE_MODE);
 				CompareEditorInput input;
@@ -572,37 +574,22 @@ public class RebaseResultDialog extends MessageDialog {
 					if (dlg.open() != Window.OK)
 						return;
 					input = new GitMergeEditorInput(dlg.useWorkspace(),
-							resources);
+							locations);
 				} else {
 					boolean useWorkspace = mergeMode == 1;
-					input = new GitMergeEditorInput(useWorkspace, resources);
+					input = new GitMergeEditorInput(useWorkspace,
+							locations);
 				}
 				CompareUI.openCompareEditor(input);
 				return;
 			} else if (skipCommitButton.getSelection()) {
 				// skip the rebase
-				final RebaseOperation op = new RebaseOperation(repo,
-						Operation.SKIP);
-				JobUtil.scheduleUserJob(op,
-						UIText.RebaseResultDialog_JobNameSkipCommit,
-						JobFamilies.REBASE, new JobChangeAdapter() {
-							@Override
-							public void done(IJobChangeEvent event) {
-								show(op.getResult(), repo);
-							}
-						});
+				SkipRebaseCommand skipCommand = new SkipRebaseCommand();
+				execute(skipCommand);
 			} else if (abortRebaseButton.getSelection()) {
 				// abort the rebase
-				final RebaseOperation op = new RebaseOperation(repo,
-						Operation.ABORT);
-				JobUtil.scheduleUserJob(op,
-						UIText.RebaseResultDialog_JobNameAbortRebase,
-						JobFamilies.REBASE, new JobChangeAdapter() {
-							@Override
-							public void done(IJobChangeEvent event) {
-								show(op.getResult(), repo);
-							}
-						});
+				AbortRebaseCommand abortCommand = new AbortRebaseCommand();
+				execute(abortCommand);
 			} else if (doNothingButton.getSelection()) {
 				// nothing
 			}
@@ -610,6 +597,14 @@ public class RebaseResultDialog extends MessageDialog {
 		super.buttonPressed(buttonId);
 	}
 
+	private void execute(AbstractRebaseCommandHandler command) {
+		try {
+			command.execute(repo);
+		} catch (ExecutionException e) {
+			Activator.showError(e.getMessage(), e);
+		}
+	}
+
 	private void createToggleButton(Composite parent) {
 		boolean toggleState = !Activator.getDefault().getPreferenceStore()
 				.getBoolean(UIPreferences.SHOW_REBASE_CONFIRM);
@@ -617,4 +612,4 @@ public class RebaseResultDialog extends MessageDialog {
 		toggleButton.setText(UIText.RebaseResultDialog_ToggleShowButton);
 		toggleButton.setSelection(toggleState);
 	}
-}
\ No newline at end of file
+}
