@@ -337,8 +337,8 @@ class ExistingOrNewPage extends WizardPage {
 					updateProjectTreeItem(treeItem, project);
 					treeItem.setText(1, project.getLocation().toOSString());
 					fillTreeItemWithGitDirectory(m, treeItem, false);
-					treeItem.setData(new ProjectAndRepo(project, treeItem
-							.getText(2)));
+					treeItem.setData(new ProjectAndRepo(project, m
+							.getGitDirAbsolutePath().toOSString()));
 					treeItem.setChecked(true);
 				}
 
@@ -351,15 +351,16 @@ class ExistingOrNewPage extends WizardPage {
 					TreeItem treeItem2 = new TreeItem(treeItem, SWT.NONE);
 					updateProjectTreeItem(treeItem2, project);
 					fillTreeItemWithGitDirectory(m, treeItem2, true);
-					treeItem2.setData(new ProjectAndRepo(project, treeItem2
-							.getText(2)));
+					treeItem2.setData(new ProjectAndRepo(project, m
+							.getGitDirAbsolutePath().toOSString()));
 					while (mi.hasNext()) { // fill in additional mappings
 						m = mi.next();
 						treeItem2 = new TreeItem(treeItem, SWT.NONE);
 						updateProjectTreeItem(treeItem2, project);
 						fillTreeItemWithGitDirectory(m, treeItem2, true);
 						treeItem2.setData(new ProjectAndRepo(m.getContainer()
-								.getProject(), treeItem2.getText(2)));
+								.getProject(), m.getGitDirAbsolutePath()
+								.toOSString()));
 					}
 					treeItem.setExpanded(true);
 					allProjectsInExistingRepos = false;
@@ -499,10 +500,7 @@ class ExistingOrNewPage extends WizardPage {
 			treeItem.setText(2,
 					UIText.ExistingOrNewPage_SymbolicValueEmptyMapping);
 		else {
-			IPath container = m.getContainerPath();
-			if (!container.isEmpty())
-				container = Path.fromOSString("."); //$NON-NLS-1$
-			IPath relativePath = container.append(m.getGitDir());
+			IPath relativePath = new Path(m.getGitDir());
 			if (isAlternative) {
 				IPath withoutLastSegment = relativePath.removeLastSegments(1);
 				IPath path;
