@@ -9,8 +9,6 @@
 package org.eclipse.egit.ui.internal.staging;
 
 import static org.eclipse.egit.ui.internal.CommonUtils.runCommand;
-import static org.eclipse.ui.ISources.ACTIVE_MENU_SELECTION_NAME;
-import static org.eclipse.ui.menus.CommandContributionItem.STYLE_PUSH;
 
 import java.io.File;
 import java.text.MessageFormat;
@@ -22,8 +20,8 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 
+import org.eclipse.core.commands.ExecutionException;
 import org.eclipse.core.commands.operations.IUndoContext;
-import org.eclipse.core.expressions.IEvaluationContext;
 import org.eclipse.core.resources.IContainer;
 import org.eclipse.core.resources.IFile;
 import org.eclipse.core.resources.IProject;
@@ -33,9 +31,7 @@ import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IAdaptable;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.core.runtime.jobs.IJobChangeEvent;
 import org.eclipse.core.runtime.jobs.Job;
-import org.eclipse.core.runtime.jobs.JobChangeAdapter;
 import org.eclipse.core.runtime.preferences.IEclipsePreferences.IPreferenceChangeListener;
 import org.eclipse.core.runtime.preferences.IEclipsePreferences.PreferenceChangeEvent;
 import org.eclipse.core.runtime.preferences.InstanceScope;
@@ -44,19 +40,21 @@ import org.eclipse.egit.core.RepositoryUtil;
 import org.eclipse.egit.core.internal.indexdiff.IndexDiffCacheEntry;
 import org.eclipse.egit.core.internal.indexdiff.IndexDiffChangedListener;
 import org.eclipse.egit.core.internal.indexdiff.IndexDiffData;
-import org.eclipse.egit.core.internal.job.JobUtil;
 import org.eclipse.egit.core.op.CommitOperation;
-import org.eclipse.egit.core.op.RebaseOperation;
 import org.eclipse.egit.core.project.RepositoryMapping;
 import org.eclipse.egit.ui.Activator;
-import org.eclipse.egit.ui.JobFamilies;
 import org.eclipse.egit.ui.UIPreferences;
 import org.eclipse.egit.ui.UIUtils;
+import org.eclipse.egit.ui.internal.CommonUtils;
 import org.eclipse.egit.ui.internal.EgitUiEditorUtils;
 import org.eclipse.egit.ui.internal.UIIcons;
 import org.eclipse.egit.ui.internal.UIText;
 import org.eclipse.egit.ui.internal.actions.ActionCommands;
 import org.eclipse.egit.ui.internal.actions.BooleanPrefAction;
+import org.eclipse.egit.ui.internal.commands.shared.AbortRebaseCommand;
+import org.eclipse.egit.ui.internal.commands.shared.AbstractRebaseCommandHandler;
+import org.eclipse.egit.ui.internal.commands.shared.ContinueRebaseCommand;
+import org.eclipse.egit.ui.internal.commands.shared.SkipRebaseCommand;
 import org.eclipse.egit.ui.internal.commit.CommitHelper;
 import org.eclipse.egit.ui.internal.commit.CommitJob;
 import org.eclipse.egit.ui.internal.commit.CommitMessageHistory;
@@ -72,7 +70,6 @@ import org.eclipse.egit.ui.internal.dialogs.SpellcheckableMessageArea;
 import org.eclipse.egit.ui.internal.gerrit.GerritUtil;
 import org.eclipse.egit.ui.internal.operations.DeletePathsOperationUI;
 import org.eclipse.egit.ui.internal.operations.IgnoreOperationUI;
-import org.eclipse.egit.ui.internal.rebase.RebaseResultDialog;
 import org.eclipse.egit.ui.internal.repository.tree.RepositoryTreeNode;
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.ControlContribution;
@@ -113,8 +110,6 @@ import org.eclipse.jface.viewers.ViewerFilter;
 import org.eclipse.jgit.api.AddCommand;
 import org.eclipse.jgit.api.CheckoutCommand;
 import org.eclipse.jgit.api.Git;
-import org.eclipse.jgit.api.RebaseCommand;
-import org.eclipse.jgit.api.RebaseCommand.Operation;
 import org.eclipse.jgit.api.ResetCommand;
 import org.eclipse.jgit.api.RmCommand;
 import org.eclipse.jgit.api.errors.GitAPIException;
@@ -165,7 +160,6 @@ import org.eclipse.ui.IEditorPart;
 import org.eclipse.ui.IFileEditorInput;
 import org.eclipse.ui.ISelectionListener;
 import org.eclipse.ui.ISelectionService;
-import org.eclipse.ui.IWorkbench;
 import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchPart;
 import org.eclipse.ui.IWorkbenchPartSite;
@@ -178,8 +172,6 @@ import org.eclipse.ui.forms.widgets.Form;
 import org.eclipse.ui.forms.widgets.FormToolkit;
 import org.eclipse.ui.forms.widgets.Section;
 import org.eclipse.ui.handlers.IHandlerService;
-import org.eclipse.ui.menus.CommandContributionItem;
-import org.eclipse.ui.menus.CommandContributionItemParameter;
 import org.eclipse.ui.operations.UndoRedoActionGroup;
 import org.eclipse.ui.part.IShowInSource;
 import org.eclipse.ui.part.ShowInContext;
@@ -330,7 +322,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 		}
 	}
 
-	class TreeDecoratingLabelProvider extends DecoratingLabelProvider {
+	static class TreeDecoratingLabelProvider extends DecoratingLabelProvider {
 
 		ILabelProvider provider;
 
@@ -377,7 +369,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 			synchronized (lock) {
 				if (currentThreadIx < globalThreadIndex)
 					return;
-				stagingView.refreshViewers();
+				stagingView.refreshViewersPreservingExpandedElements();
 			}
 		}
 
@@ -584,7 +576,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 		rebaseContinueButton.setImage(getImage(UIIcons.REBASE_CONTINUE));
 		buttonGridData.applyTo(rebaseContinueButton);
 
-		updateRebaseButtonVisibility(false);
+		showControl(rebaseSection, false);
 
 		commitMessageSection = toolkit.createSection(rebaseAndCommitComposite,
 				ExpandableComposite.TITLE_BAR);
@@ -783,14 +775,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 						event.detail = DND.DROP_COPY;
 						if (event.data instanceof IStructuredSelection) {
 							final IStructuredSelection selection = (IStructuredSelection) event.data;
-							Object firstElement = selection.getFirstElement();
-							if (firstElement instanceof StagingEntry)
-								stage(selection);
-							else {
-								IResource resource = AdapterUtils.adapt(firstElement, IResource.class);
-								if (resource != null)
-									stage(selection);
-							}
+							stage(selection);
 						}
 					}
 
@@ -837,6 +822,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 		updateSectionText();
 		updateToolbar();
 		enableCommitWidgets(false);
+		refreshAction.setEnabled(false);
 
 		createPopupMenu(unstagedViewer);
 		createPopupMenu(stagedViewer);
@@ -912,41 +898,36 @@ public class StagingView extends ViewPart implements IShowInSource {
 		stagedViewer.addFilter(filter);
 	}
 
-	private void doRebaseOperation(Repository repository,
-			RebaseCommand.Operation operation, String jobname) {
-		final RebaseOperation op = new RebaseOperation(repository, operation);
-		JobUtil.scheduleUserJob(op, jobname, JobFamilies.REBASE,
-				new JobChangeAdapter() {
-					@Override
-					public void done(IJobChangeEvent event) {
-						RebaseResultDialog.show(op.getResult(),
-								currentRepository);
-					}
-				});
+	private void executeRebaseOperation(AbstractRebaseCommandHandler command) {
+		try {
+			command.execute(currentRepository);
+		} catch (ExecutionException e) {
+			Activator.showError(e.getMessage(), e);
+		}
 	}
 
 	/**
 	 * Abort rebase command in progress
 	 */
 	protected void rebaseAbort() {
-		doRebaseOperation(currentRepository, Operation.ABORT,
-				UIText.AbortRebaseCommand_JobName);
+		AbortRebaseCommand abortCommand = new AbortRebaseCommand();
+		executeRebaseOperation(abortCommand);
 	}
 
 	/**
 	 * Rebase next commit and continue rebase in progress
 	 */
 	protected void rebaseSkip() {
-		doRebaseOperation(currentRepository, Operation.SKIP,
-				UIText.SkipRebaseCommand_JobName);
+		SkipRebaseCommand skipCommand = new SkipRebaseCommand();
+		executeRebaseOperation(skipCommand);
 	}
 
 	/**
 	 * Continue rebase command in progress
 	 */
 	protected void rebaseContinue() {
-		doRebaseOperation(currentRepository, Operation.CONTINUE,
-				UIText.ContinueRebaseCommand_JobName);
+		ContinueRebaseCommand continueCommand = new ContinueRebaseCommand();
+		executeRebaseOperation(continueCommand);
 	}
 
 	private void createUnstagedToolBarComposite() {
@@ -1096,21 +1077,20 @@ public class StagingView extends ViewPart implements IShowInSource {
 	}
 
 	private void enableCommitWidgets(boolean enabled) {
-		if (!enabled) {
-			commitMessageText.setText(""); //$NON-NLS-1$
-			committerText.setText(""); //$NON-NLS-1$
-			authorText.setText(""); //$NON-NLS-1$
-		}
-
 		commitMessageText.setEnabled(enabled);
 		committerText.setEnabled(enabled);
 		enableAuthorText(enabled);
-		refreshAction.setEnabled(enabled);
 		amendPreviousCommitAction.setEnabled(enabled);
 		signedOffByAction.setEnabled(enabled);
 		addChangeIdAction.setEnabled(enabled);
 		commitButton.setEnabled(enabled);
 		commitAndPushButton.setEnabled(enabled);
+
+		if (!enabled) {
+			commitMessageText.setText(""); //$NON-NLS-1$
+			committerText.setText(""); //$NON-NLS-1$
+			authorText.setText(""); //$NON-NLS-1$
+		}
 	}
 
 	private void enableAuthorText(boolean enabled) {
@@ -1223,7 +1203,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 				final boolean enable = isChecked();
 				getLabelProvider(stagedViewer).setFileNameMode(enable);
 				getLabelProvider(unstagedViewer).setFileNameMode(enable);
-				refreshViewers();
+				refreshViewersPreservingExpandedElements();
 				getPreferenceStore().setValue(
 						UIPreferences.STAGING_VIEW_FILENAME_MODE, enable);
 			}
@@ -1237,6 +1217,8 @@ public class StagingView extends ViewPart implements IShowInSource {
 		listPresentationAction = new Action(UIText.StagingView_List,
 				IAction.AS_RADIO_BUTTON) {
 			public void run() {
+				if (!isChecked())
+					return;
 				presentation = Presentation.LIST;
 				getPreferenceStore().setValue(
 						UIPreferences.STAGING_VIEW_PRESENTATION,
@@ -1253,6 +1235,8 @@ public class StagingView extends ViewPart implements IShowInSource {
 		treePresentationAction = new Action(UIText.StagingView_Tree,
 				IAction.AS_RADIO_BUTTON) {
 			public void run() {
+				if (!isChecked())
+					return;
 				presentation = Presentation.TREE;
 				getPreferenceStore().setValue(
 						UIPreferences.STAGING_VIEW_PRESENTATION,
@@ -1269,6 +1253,8 @@ public class StagingView extends ViewPart implements IShowInSource {
 		compactTreePresentationAction = new Action(UIText.StagingView_CompactTree,
 				IAction.AS_RADIO_BUTTON) {
 			public void run() {
+				if (!isChecked())
+					return;
 				presentation = Presentation.COMPACT_TREE;
 				getPreferenceStore().setValue(
 						UIPreferences.STAGING_VIEW_PRESENTATION,
@@ -1516,6 +1502,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 								fileSelection, false));
 					else
 						menuMgr.add(createItem(
+								UIText.StagingView_replaceWithFileInGitIndex,
 								ActionCommands.DISCARD_CHANGES_ACTION,
 								fileSelection)); // replace with index
 				if (addReplaceWithHeadRevision)
@@ -1525,6 +1512,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 								fileSelection, true));
 					else
 						menuMgr.add(createItem(
+								UIText.StagingView_replaceWithHeadRevision,
 								ActionCommands.REPLACE_WITH_HEAD_ACTION,
 								fileSelection));
 				if (addIgnore)
@@ -1532,7 +1520,8 @@ public class StagingView extends ViewPart implements IShowInSource {
 				if (addDelete)
 					menuMgr.add(new DeleteAction(fileSelection));
 				if (addLaunchMergeTool)
-					menuMgr.add(createItem(ActionCommands.MERGE_TOOL_ACTION,
+					menuMgr.add(createItem(UIText.StagingView_MergeTool,
+							ActionCommands.MERGE_TOOL_ACTION,
 							fileSelection));
 				menuMgr.add(new Separator());
 				menuMgr.add(createShowInMenu());
@@ -1553,30 +1542,46 @@ public class StagingView extends ViewPart implements IShowInSource {
 	 *         no filter
 	 */
 	String getFilterString() {
-		if (filterText != null)
+		if (filterText != null && !filterText.isDisposed())
 			return filterText.getText().trim();
 		else
 			return ""; //$NON-NLS-1$
 	}
 
 	/**
-	 * Refresh the unstaged and staged viewers
+	 * Refresh the unstaged and staged viewers without preserving expanded
+	 * elements
 	 */
 	public void refreshViewers() {
+		Display.getDefault().syncExec(new Runnable() {
+			public void run() {
+				refreshViewersInternal();
+			}
+		});
+	}
+
+	/**
+	 * Refresh the unstaged and staged viewers, preserving expanded elements
+	 */
+	public void refreshViewersPreservingExpandedElements() {
 		Display.getDefault().syncExec(new Runnable() {
 			public void run() {
 				Object[] unstagedExpanded = unstagedViewer
 						.getExpandedElements();
 				Object[] stagedExpanded = stagedViewer.getExpandedElements();
-				unstagedViewer.refresh();
-				stagedViewer.refresh();
-				updateSectionText();
+				refreshViewersInternal();
 				unstagedViewer.setExpandedElements(unstagedExpanded);
 				stagedViewer.setExpandedElements(stagedExpanded);
 			}
 		});
 	}
 
+	private void refreshViewersInternal() {
+		unstagedViewer.refresh();
+		stagedViewer.refresh();
+		updateSectionText();
+	}
+
 	private IContributionItem createShowInMenu() {
 		IWorkbenchWindow workbenchWindow = getSite().getWorkbenchWindow();
 		return UIUtils.createShowInMenu(workbenchWindow);
@@ -1772,20 +1777,14 @@ public class StagingView extends ViewPart implements IShowInSource {
 		return availableActions;
 	}
 
-	private CommandContributionItem createItem(String itemAction,
-			final ISelection selection) {
-		IWorkbench workbench = PlatformUI.getWorkbench();
-		CommandContributionItemParameter itemParam = new CommandContributionItemParameter(
-				workbench, null, itemAction, STYLE_PUSH);
-
-		IWorkbenchWindow activeWorkbenchWindow = workbench
-				.getActiveWorkbenchWindow();
-		IHandlerService hsr = (IHandlerService) activeWorkbenchWindow
-				.getService(IHandlerService.class);
-		IEvaluationContext ctx = hsr.getCurrentState();
-		ctx.addVariable(ACTIVE_MENU_SELECTION_NAME, selection);
-
-		return new CommandContributionItem(itemParam);
+	private IAction createItem(String text, final String commandId,
+			final IStructuredSelection selection) {
+		return new Action(text) {
+			@Override
+			public void run() {
+				CommonUtils.runCommand(commandId, selection);
+			}
+		};
 	}
 
 	private void reactOnSelection(ISelection selection) {
@@ -2000,6 +1999,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 		unstagedViewer.setInput(update);
 		stagedViewer.setInput(update);
 		enableCommitWidgets(false);
+		refreshAction.setEnabled(false);
 		updateSectionText();
 		form.setText(UIText.StagingView_NoSelectionTitle);
 	}
@@ -2016,16 +2016,33 @@ public class StagingView extends ViewPart implements IShowInSource {
 				showControl(rebaseSection, isRebasing);
 				rebaseSection.getParent().layout(true);
 			}
-
-			private void showControl(Control c, final boolean show) {
-				c.setVisible(show);
-				GridData g = (GridData) c.getLayoutData();
-				g.exclude = !show;
-			}
 		});
 	}
 
-	private void reload(final Repository repository) {
+	private static void showControl(Control c, final boolean show) {
+		c.setVisible(show);
+		GridData g = (GridData) c.getLayoutData();
+		g.exclude = !show;
+	}
+
+	/**
+	 * @param isAmending
+	 *            if the current commit should be amended
+	 */
+	public void setAmending(boolean isAmending) {
+		if (amendPreviousCommitAction.isChecked() != isAmending) {
+			amendPreviousCommitAction.setChecked(isAmending);
+			amendPreviousCommitAction.run();
+		}
+
+	}
+
+	/**
+	 * Reload the staging view
+	 *
+	 * @param repository
+	 */
+	public void reload(final Repository repository) {
 		if (form.isDisposed())
 			return;
 		if (repository == null) {
@@ -2077,8 +2094,7 @@ public class StagingView extends ViewPart implements IShowInSource {
 						pathsToExpandInUnstaged);
 				expandPreviousExpandedAndPaths(stagedExpanded, stagedViewer,
 						pathsToExpandInStaged);
-				enableCommitWidgets(indexDiffAvailable
-						&& indexDiff.getConflicting().isEmpty());
+				refreshAction.setEnabled(true);
 
 				updateRebaseButtonVisibility(repository.getRepositoryState()
 						.isRebasing());
@@ -2099,6 +2115,8 @@ public class StagingView extends ViewPart implements IShowInSource {
 
 				form.setText(StagingView.getRepositoryName(repository));
 				updateCommitMessageComponent(repositoryChanged, indexDiffAvailable);
+				enableCommitWidgets(indexDiffAvailable
+						&& indexDiff.getConflicting().isEmpty());
 				updateSectionText();
 			}
 		});
@@ -2124,6 +2142,10 @@ public class StagingView extends ViewPart implements IShowInSource {
 		if (viewer.getAutoExpandLevel() == AbstractTreeViewer.ALL_LEVELS)
 			return;
 
+		// No need to expand anything
+		if (getPresentation() == Presentation.LIST)
+			return;
+
 		Set<IPath> paths = new HashSet<IPath>(additionalPaths);
 		// Instead of just expanding the previous elements directly, also expand
 		// all parent paths. This makes it work in case of "re-folding" of
@@ -2131,16 +2153,31 @@ public class StagingView extends ViewPart implements IShowInSource {
 		for (Object element : previous)
 			if (element instanceof StagingFolderEntry)
 				addPathAndParentPaths(((StagingFolderEntry) element).getPath(), paths);
-		List<Object> expand = new ArrayList<Object>();
+		List<StagingFolderEntry> expand = new ArrayList<StagingFolderEntry>();
 		StagingViewContentProvider stagedContentProvider = getContentProvider(viewer);
-		for (StagingFolderEntry folder : stagedContentProvider
-				.getStagingFolderEntries()) {
-			if (paths.contains(folder.getPath()))
-				expand.add(folder);
-		}
+		calculateNodesToExpand(paths, stagedContentProvider.getElements(null),
+				expand);
 		viewer.setExpandedElements(expand.toArray());
 	}
 
+	private void calculateNodesToExpand(Set<IPath> paths, Object[] elements,
+			List<StagingFolderEntry> result) {
+		if (elements == null)
+			return;
+
+		for (Object element : elements) {
+			if (element instanceof StagingFolderEntry) {
+				StagingFolderEntry folder = (StagingFolderEntry) element;
+				if (paths.contains(folder.getPath())) {
+					result.add(folder);
+					// Only recurs if folder matched (i.e. don't try to expand
+					// children of unexpanded parents)
+					calculateNodesToExpand(paths, folder.getChildren(), result);
+				}
+			}
+		}
+	}
+
 	private void clearCommitMessageToggles() {
 		amendPreviousCommitAction.setChecked(false);
 		addChangeIdAction.setChecked(false);
@@ -2205,9 +2242,11 @@ public class StagingView extends ViewPart implements IShowInSource {
 	}
 
 	private void addHeadChangedWarning(String commitMessage) {
-		String message = UIText.StagingView_headCommitChanged + Text.DELIMITER
-				+ Text.DELIMITER + commitMessage;
-		commitMessageComponent.setCommitMessage(message);
+		if (!commitMessage.startsWith(UIText.StagingView_headCommitChanged)) {
+			String message = UIText.StagingView_headCommitChanged
+					+ Text.DELIMITER + Text.DELIMITER + commitMessage;
+			commitMessageComponent.setCommitMessage(message);
+		}
 	}
 
 	private void loadInitialState(CommitHelper helper) {
